import logging
from os import path
from src.api import oking
import sys

global is_connected_oracle_client, client_data

logging.basicConfig(level=logging.DEBUG,
                    format='[%(levelname)s][%(asctime)s] %(message)s',
                    datefmt='%Y-%m-%d %I:%M:%S')
logger = logging.getLogger()


def print_version():
    f = open(path.abspath('version.txt'), 'r')
    v = f.read()
    print(v)
    f.close()
    exit(0)


def get_token_from_params(args: list) -> str:
    if len(args) >= 2:
        if args[1] == '--version':
            print_version()
        return args[1]
    else:
        logger.error('Informe o token da integracao como parametro')
        exit(1)


token_oking = get_token_from_params(sys.argv)
logger.info('Iniciando oking __init__')

# Consultar dados da integracao do cliente (modulos, tempo de execucao, dados api okvendas)
client_data = oking.get(f'https://appbuilder.openk.com.br/api/consulta/integracao_oking/filtros?token={token_oking}', None)

if client_data is not None:
    assert client_data['integracao_id'] is not None, 'Id da integracao nao informado (Api Oking)'
    assert client_data['db_type'] is not None, 'Tipo do banco de dados nao informado (Api Oking)'
    assert client_data['host'] is not None, 'Host do banco de dados nao informado (Api Oking)'
    assert client_data['database'] is not None, 'Nome do banco de dados nao informado (Api Oking)'
    assert client_data['user'] is not None, 'Usuario do banco de dados nao informado (Api Oking)'
    assert client_data['password'] is not None, 'Senha do banco de dados nao informado (Api Oking)'
    assert client_data['url_api'] is not None, 'Url da api okvendas nao informado (Api Oking)'
    assert client_data['token_api'] is not None, 'Token da api okvendas nao informado (Api Oking)'
    is_connected_oracle_client = False
else:
    logger.warning(f'Cliente nao configurado no painel oking para o token: {token_oking}')



import requests
import logging

logger = logging.getLogger()


def register_warn(message):
    logger.warning(message)
    post_message(message)


def register_error(erro):
    logger.error(erro)
    post_message(erro)


def post_message(message):
    dct_body = {
        'text': message,
        'channel': '#src'
    }

    url = 'https://hooks.slack.com/services/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'

    try:
        headers = {'Content-type': 'application/json',
                   'Accept': 'text/html'
                   }
        response = requests.post(url, json=dct_body, headers=headers)

        if response.status_code >= 200 and response.status_code <= 299:
            logger.info('Envio de mensagem para o slack')
        else:
            msg = 'Erro ao enviar slack ' + \
                response.status_code + ' ' + dct_body
            logger.error(msg)
            response.raise_for_status()

    except Exception as ex:
        logger.error(str(ex), exc_info=True)





class CatalogoResponse:
    def __init__(self, Status: int, Message: str, codigo_erp: str = '', protocolo: str = '', loja: str = '') -> None:
        self.codigo_erp = codigo_erp
        self.status = Status
        self.message = Message
        self.protocolo = protocolo
        self.loja = loja


class PriceResponse:
    def __init__(self, codigo_erp, Status: int, Message: str, protocolo: str, loja: str):
        self.codigo_erp = codigo_erp
        self.status = Status
        self.message = Message
        self.protocolo = protocolo


class InvoiceResponse:
    def __init__(self, Status: int, Message: str):
        self.status = Status
        self.message = Message


class TrackingResponse:
    def __init__(self, Status: int, Message: str):
        self.status = Status
        self.message = Message


import logging
import src.database.connection as database
import src.database.utils as utils
from src.database.utils import DatabaseConfig
import src.api.okvendas as api_okvendas
from src.api import slack
from src.database import queries
import src

logger = logging.getLogger()


def job_insert_stock_semaphore(job_config_dict: dict):
    """
    Job para inserir estoques no banco semáforo
    Args:
        job_config_dict: Configuração do job
    """
    db_config = utils.get_database_config(job_config_dict)
    if db_config.sql is None:
        logger.warning(job_config_dict.get('job_name') + ' | Comando sql para inserir estoques no semaforo nao encontrado')
    else:
        db = database.Connection(db_config)
        conn = db.get_conect()
        cursor = conn.cursor()

        try:
            logger.info(job_config_dict.get('job_name') + ' | Inserindo estoques no banco semaforo')
            logger.info(db_config.sql)
            cursor.execute(db_config.sql)
            logger.info(job_config_dict.get('job_name') + f' | {cursor.rowcount} estoques inseridos no banco semaforo')
        except Exception as ex:
            logger.error(job_config_dict.get('job_name') + ' | Erro ' + str(ex), exc_info=True)

        cursor.close()
        conn.commit()
        conn.close()


def job_send_stocks(job_config_dict: dict):
    """
    Job para realizar a atualização dos estoques padrão
    Args:
        job_config_dict: Configuração do job
    """
    db_config = utils.get_database_config(job_config_dict)
    stocks = query_stocks(db_config)
    p_size = len(stocks) if stocks is not None else 0
    atualizados = []
    if p_size > 0:
        logger.debug("Total de produtos atualiza estoque API: {}".format(p_size))
        for stock in stocks:
            try:
                response, status_code = api_okvendas.send_stocks(src.client_data.get('url_api') + '/catalogo/estoque', stock, src.client_data.get('token_api'))

                if response is not None:
                    logger.debug('Atualizando produtos no semaforo')
                    conexao = database.Connection(db_config)
                    conn = conexao.get_conect()
                    cursor = conn.cursor()

                    codigo_erp = response['codigo_erp']
                    if response['Status'] == 1 or response['Status'] == 'Success':
                        try:
                            sql_protocolar_estoque = queries.get_stock_protocol_command(db_config.db_type)
                            cursor.execute(sql_protocolar_estoque, queries.get_command_parameter(db_config.db_type, [codigo_erp]))
                            atualizados.append(response['codigo_erp'])
                        except Exception as ex:
                            logger.error(str(ex), exc_info=True)
                    else:
                        logger.warning(job_config_dict.get('job_name') + f' | Erro ao atualizar estoque para o sku {codigo_erp}: {response["Message"]}')

                    cursor.close()
                    conn.commit()
                    conn.close()
            except Exception as e:
                logger.warning(job_config_dict.get('job_name') + f' | Erro ao atualizar estoque para o sku {stock.get("codigo_erp")}: {str(e)}')

        total = len(atualizados)
        logger.debug('Atualizado estoques no semaforo: {}'.format(total))
    else:
        logger.warning("Nao ha produtos para atualizar estoque")


def job_send_stocks_ud(job_config_dict: dict):
    """
    Job para realizar a atualização dos estoques por unidade de distribuição
    Args:
        job_config_dict: Configuração do job
    """
    db_config = utils.get_database_config(job_config_dict)
    stocks = query_stocks_ud(db_config)
    p_size = len(stocks) if stocks is not None else 0

    if p_size > 0:
        logger.debug("Total de produtos atualiza estoque API: {}".format(p_size))
        jsn_prods = api_okvendas.send_stocks(src.client_data.get('url_api') + '/catalogo/estoqueUnidadeDistribuicao', stocks, src.client_data.get('token_api'))

        atualizados = []

        if jsn_prods is not None:
            logger.debug('Atualizando produtos no semaforo')
            conexao = database.Connection(db_config)
            conn = conexao.get_conect()
            cursor = conn.cursor()

            # Identifiers, Status, Message, Protocolo
            for p in jsn_prods:
                try:
                    if p['Status'] == 1:
                        cod_erp = p['Identifiers'][0]
                        sql_protocolar_estoque = queries.get_stock_protocol_command(db_config.db_type)
                        cursor.execute(sql_protocolar_estoque, queries.get_command_parameter(db_config.db_type, [cod_erp]))
                        atualizados.append(cod_erp)
                    else:
                        logger.error(f'Erro ao atualizar estoque {p["Identifiers"][0]} erro da api: ' + p['Message'])
                except Exception as e:
                    logger.error(f'Erro ao protocolar estoque do sku {p["Identifiers"][0]}: {str(e)}')

            cursor.close()
            conn.commit()
            conn.close()

            total = len(atualizados)
            logger.debug('Atualizado produtos no semaforo: {}'.format(total))
    else:
        logger.warning("Nao ha produtos para atualizar estoque")


def query_stocks_ud(db_config: DatabaseConfig):
    """
    Consulta no banco de dados os estoques pendentes de atualização por unidade de distribuição
    Args:
        db_config: Configuração do banco de dados

    Returns:
    Lista de estoques para realizar a atualização
    """
    stocks = None
    if db_config.sql is None:
        slack.register_warn("Query estoque de produtos nao configurada!")
    else:
        try:
            conexao = database.Connection(db_config)
            conn = conexao.get_conect()
            cursor = conn.cursor()

            # print(db_config.sql)
            cursor.execute(db_config.sql)
            rows = cursor.fetchall()
            # print(rows)
            columns = [col[0] for col in cursor.description]
            results = [dict(zip(columns, row)) for row in rows]

            cursor.close()
            conn.close()
            if len(results) > 0:
                stocks = stock_ud_dict(results)

        except Exception as ex:
            logger.error(str(ex), exc_info=True)

    return stocks


def query_stocks(db_config: DatabaseConfig):
    """
    Consulta no banco de dados os estoques pendentes de atualização padrão
    Args:
        db_config: Configuração do banco de dados

    Returns:
    Lista de estoques para realizar a atualização
    """
    produtos = None
    if db_config.sql is None:
        slack.register_warn("Query estoque de produtos nao configurada!")
    else:
        try:
            conexao = database.Connection(db_config)
            conn = conexao.get_conect()
            cursor = conn.cursor()

            # print(db_config.sql)
            cursor.execute(db_config.sql)
            rows = cursor.fetchall()
            # print(rows)
            columns = [col[0] for col in cursor.description]
            results = [dict(zip(columns, row)) for row in rows]

            cursor.close()
            conn.close()
            if len(results) > 0:
                produtos = stock_dict(results)

        except Exception as ex:
            logger.error(str(ex), exc_info=True)

    return produtos


def stock_dict(produtos):
    lista = []
    for row in produtos:
        pdict = {
            'codigo_erp': row['CODIGO_ERP'],
            'quantidade': int(row['QUANTIDADE'])
        }
        lista.append(pdict)

    return lista


def stock_ud_dict(produtos):
    lista = []
    for row in produtos:
        pdict = {
            'unidade_distribuicao': row[0],
            'codigo_erp': row[1],
            'quantidade_total': int(row[2]),
            'parceiro': 1
        }
        lista.append(pdict)

    return lista


import src
from time import sleep
import schedule
import logging
from src.jobs import products_jobs
from src.jobs import stock_jobs
from src.jobs import price_jobs
from src.jobs import order_jobs
from src.api import oking

logger = logging.getLogger()


# region Estoques

def instantiate_insert_stock_semaphore_job(job_config: dict) -> None:
    """
    Instancia o job de inserção/atualização de estoques na tabela de semáforo
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(job_config.get('job_name') + ' | Iniciando execucao')
    stock_jobs.job_insert_stock_semaphore(job_config)


def instantiate_send_stock_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos estoques para a api okVendas
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(job_config.get('job_name') + ' | Iniciando execucao')
    stock_jobs.job_send_stocks(job_config)


def instantiate_send_ud_stock_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos estoques por Unidades de distribuição para a api okVendas
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(job_config.get('job_name') + ' | Iniciando execucao')
    stock_jobs.job_send_stocks_ud(job_config)


# endregion Estoques


# region Precos

def instantiate_insert_price_semaphore_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos preços para o banco semáforo
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(job_config.get('job_name') + ' | Iniciando execucao')
    price_jobs.job_insert_prices_semaphore(job_config)


def instantiate_send_price_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos preços para a api okVendas
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(job_config.get('job_name') + ' | Iniciando execucao')
    price_jobs.job_send_prices(job_config)


# endregion Precos


# region Produtos

def instantiate_insert_products_semaphore_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos produtos para o banco semáforo
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(job_config.get('job_name') + ' | Iniciando execucao')
    products_jobs.job_insert_products_semaphore(job_config)


def instantiate_update_products_semaphore_job(job_config: dict) -> None:
    """
    Instancia o job de atualização dos produtos no banco semáforo
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(job_config.get('job_name') + ' | Iniciando execucao')
    products_jobs.job_update_products_semaphore(job_config)


def instantiate_send_products_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos produtos para a api okVendas
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(job_config.get('job_name') + ' | Iniciando execucao')
    products_jobs.job_send_products(job_config)


# endregion Produtos


# region Pedidos

def instantiate_orders_job(job_config: dict) -> None:
    """
    Instancia o job de envio dos pedidos para o ERP
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(job_config.get('job_name') + ' | Iniciando execucao')
    order_jobs.define_job_start(job_config)


def instantiate_invoice_job(job_config: dict) -> None:
    """
    Intancia o job de envio das NFs de pedidos para a api okvendas
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(job_config.get('job_name') + ' | Iniciando execucao')
    order_jobs.job_invoice_orders(job_config)


def instantiate_erp_tracking_job(job_config: dict) -> None:
    """
    Intancia o job de envio do rastreio de pedidos para a api okvendas
    Args:
        job_config: Configuração do job obtida na api do oking
    """
    logger.info(job_config.get('job_name') + ' | Iniciando execucao')
    order_jobs.job_send_erp_tracking(job_config)

# endregion Pedidos


# region ConfigJobs

def schedule_job(job_config: dict, time_unit: str, time: int) -> None:
    logger.info(f'Adicionando job {job_config.get("job_name")} ao schedule de {time} em {time} {time_unit}')
    if time_unit == 'M':  # Minutos
        func = get_job_from_name(job_config.get('job_name'))
        schedule.every(time).minutes.do(func, job_config)
    elif time_unit == 'H':  # Horas
        func = get_job_from_name(job_config.get('job_name'))
        schedule.every(time).hours.do(func, job_config)
    elif time_unit == 'D':  # Dias
        func = get_job_from_name(job_config.get('job_name'))
        schedule.every(time).days.do(func, job_config)


def get_job_from_name(job_name: str):
    if job_name == 'insere_estoque_produto_semaforo_job':
        return instantiate_insert_stock_semaphore_job
    elif job_name == 'envia_estoque_produtos_job':
        return instantiate_send_stock_job
    elif job_name == 'insere_preco_produto_semaforo_job':
        return instantiate_insert_price_semaphore_job
    elif job_name == 'envia_preco_produtos_job':
        return instantiate_send_price_job
    elif job_name == 'envia_catalogo_produto_semaforo_job':
        return instantiate_insert_products_semaphore_job
    elif job_name == 'envia_catalogo_produto_loja_job':
        return instantiate_send_products_job
    elif job_name == 'internaliza_pedidos_job':
        return instantiate_orders_job
    elif job_name == 'envia_nota_loja_job':
        return instantiate_invoice_job
    elif job_name == 'envia_rastreio_pedido_job':
        return instantiate_erp_tracking_job

# endregion ConfigJobs


modules: list = [oking.Module(**m) for m in src.client_data.get('modulos')]
assert modules is not None, 'Nao foi possivel obter os modulos da integracao. Por favor, entre em contato com o suporte.'
for module in modules:
    schedule_job({
        'db_host': src.client_data.get('host'),
        'db_user': src.client_data.get('user'),
        'db_type': src.client_data.get('db_type'),
        'db_name': src.client_data.get('database'),
        'db_pwd': src.client_data.get('password'),
        'db_client': src.client_data.get('diretorio_client'),
        'job_name': module.job_name,
        'sql': module.sql
    }, module.time_unit, module.time)


def main():
    logger.info('Iniciando oking __main__')
    while True:
        schedule.run_pending()
        sleep(5)


if __name__ == "__main__":
    main()




class Pagamento:

    def __init__(self, opcao_pagamento, parcelas, bandeira,
                 condicao_pagamento_erp, tabela_financiamento_rsvarejo,
                 tipo_venda_rsvarejo, canal_venda_id, canal_venda,
                 numero_cupom, valor_cupom, codigo_compra, codigo_pedido_canal,
                 data_movimento, codigo_mercado, titulos):
        self.opcao_pagamento = opcao_pagamento
        self.parcelas = parcelas
        self.bandeira = bandeira
        self.condicao_pagamento_erp = condicao_pagamento_erp
        self.tabela_financiamento_rsvarejo = tabela_financiamento_rsvarejo
        self.tipo_venda_rsvarejo = tipo_venda_rsvarejo
        self.canal_venda_id = canal_venda_id
        self.canal_venda = canal_venda
        self.numero_cupom = numero_cupom
        self.valor_cupom = valor_cupom
        self.codigo_compra = codigo_compra
        self.codigo_pedido_canal = codigo_pedido_canal
        self.data_movimento = data_movimento
        self.codigo_mercado = codigo_mercado
        self.titulos = titulos  # [TituloPagamento]


class TituloPagamento:

    def __init__(self, parcela, valor, data_vencimento, data_baixa,
                 data_pago, codigo_retorno, descricao_retorno,
                 valor_abatimento_concebido, desconto_concebido,
                 valor_iof_recolhido, valor_creditado,
                 valor_sacado, outros_valores, outros_creditos):
        self.parcela = parcela
        self.valor = valor
        self.data_vencimento = data_vencimento
        self.data_baixa = data_baixa
        self.data_pago = data_pago
        self.codigo_retorno = codigo_retorno
        self.descricao_retorno = descricao_retorno
        self.valor_abatimento_concebido = valor_abatimento_concebido
        self.desconto_concebido = desconto_concebido
        self.valor_iof_recolhido = valor_iof_recolhido
        self.valor_creditado = valor_creditado
        self.valor_sacado = valor_sacado
        self.outros_valores = outros_valores
        self.outros_creditos = outros_creditos


class FormaPagamento:

    def __init__(self, tipo_pagamento, bandeira, codigo_autorizacao,
                 codigo_status, codigo_transacao, data_criacao,
                 data_aprovacao, detalhe_status, status,
                 valor_frete, valor_parcela, valor_pedido,
                 valor_total_pago, parcelas):
        self.tipo_pagamento = tipo_pagamento
        self.bandeira = bandeira
        self.codigo_autorizacao = codigo_autorizacao
        self.codigo_status = codigo_status
        self.codigo_transacao = codigo_transacao
        self.data_criacao = data_criacao
        self.data_aprovacao = data_aprovacao
        self.detalhe_status = detalhe_status
        self.status = status
        self.valor_frete = valor_frete
        self.valor_parcela = valor_parcela
        self.valor_pedido = valor_pedido
        self.valor_total_pago = valor_total_pago
        self.parcelas = parcelas




# Queries nao podem terminar com ponto e virgula

def get_product_protocol_command(connection_type: str):
	if connection_type.lower() == 'mysql':
		return 'update openk_semaforo.produto set data_sincronizacao = now() where codigo_erp = %s and codigo_erp_sku = %s'
	elif connection_type.lower() == 'oracle':
		return 'update openk_semaforo.produto set data_sincronizacao = SYSDATE where codigo_erp = :codigo_erp and codigo_erp_sku = :codigo_erp_sku'
	elif connection_type.lower() == 'sqlserver':
		return 'update openk_semaforo.produto set data_sincronizacao = now() where codigo_erp = ? and codigo_erp_sku = ?'


def get_stock_protocol_command(connection_type: str):
	if connection_type.lower() == 'mysql':
		return 'update openk_semaforo.estoque_produto set data_sincronizacao = now() where codigo_erp = %s'
	elif connection_type.lower() == 'oracle':
		return 'update openk_semaforo.estoque_produto set data_sincronizacao = SYSDATE where codigo_erp = :codigo_erp'
	elif connection_type.lower() == 'sqlserver':
		return 'update openk_semaforo.estoque_produto set data_sincronizacao = getdate() where codigo_erp = ?'


def get_price_protocol_command(connection_type: str):
	if connection_type.lower() == 'mysql':
		return 'update openk_semaforo.preco_produto set data_sincronizacao = now() where codigo_erp = %s'
	elif connection_type.lower() == 'oracle':
		return 'update openk_semaforo.preco_produto set data_sincronizacao = SYSDATE where codigo_erp = :codigo_erp'
	elif connection_type.lower() == 'sqlserver':
		return 'update openk_semaforo.preco_produto set data_sincronizacao = getdate() where codigo_erp = ?'


def get_insert_client_command(connection_type: str):
	if connection_type.lower() == 'mysql':
		return '''insert into openk_semaforo.cliente (nome, razao_social, cpf, cnpj, email, telefone_residencial, telefone_celular, cep, 
														tipo_logradouro, logradouro, numero, complemento, bairro, cidade, estado, referencia)
					values (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s) '''
	elif connection_type.lower() == 'oracle':
		return '''INSERT INTO OPENK_SEMAFORO.CLIENTE (NOME, RAZAO_SOCIAL, CPF, CNPJ, EMAIL, TELEFONE_RESIDENCIAL, TELEFONE_CELULAR, CEP, 
														TIPO_LOGRADOURO, LOGRADOURO,NUMERO, COMPLEMENTO, BAIRRO, CIDADE, ESTADO, REFERENCIA) 
				VALUES (:1, :2, :3, :4, :5, :6, :7, :8, :9, :10, :11, :12, :13, :14, :15, :16) '''
	elif connection_type.lower() == 'sqlserver':
		return '''insert into openk_semaforo.cliente (nome, razao_social, cpf, cnpj, email, telefone_residencial, telefone_celular, cep, 
														tipo_logradouro, logradouro, numero, complemento, bairro, cidade, estado, referencia)
					values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?) '''


def get_insert_order_command(connection_type: str):
	if connection_type.lower() == 'mysql':
		return '''insert into openk_semaforo.pedido (pedido_id, pedido_venda_id, data_pedido, status, cliente_id, valor, valor_desconto, valor_frete, 
					valor_adicional, data_pagamento, tipo_pagamento, bandeira, parcelas, condicao_pagamento_erp, codigo_rastreio, data_previsao_entrega, 
					transportadora, modo_envio)
					values (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s) '''
	elif connection_type.lower() == 'oracle':
		return '''INSERT INTO OPENK_SEMAFORO.PEDIDO (PEDIDO_ID, PEDIDO_VENDA_ID, DATA_PEDIDO, STATUS, CLIENTE_ID, VALOR, VALOR_DESCONTO, VALOR_FRETE, 
					VALOR_ADICIONAL, DATA_PAGAMENTO, TIPO_PAGAMENTO, BANDEIRA, PARCELAS, CONDICAO_PAGAMENTO_ERP, CODIGO_RASTREIO, DATA_PREVISAO_ENTREGA, 
					TRANSPORTADORA, MODO_ENVIO)
					VALUES (:1, :2, TO_DATE(:3, 'YYYY-MM-DD HH24:MI:SS'), :4, :5, :6, :7, :8, :9, TO_DATE(:10, 'YYYY-MM-DD HH24:MI:SS'), :11, :12, :13, :14, :15, TO_DATE(:16, 'YYYY-MM-DD HH24:MI:SS'), :17, :18) '''
	elif connection_type.lower() == 'sqlserver':
		return '''insert into openk_semaforo.pedido (pedido_id, pedido_venda_id, data_pedido, status, cliente_id, valor, valor_desconto, valor_frete, 
					valor_adicional, data_pagamento, tipo_pagamento, bandeira, parcelas, condicao_pagamento_erp, codigo_rastreio, data_previsao_entrega, 
					transportadora, modo_envio)
					values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?) '''


def get_insert_order_items_command(connection_type: str):
	if connection_type.lower() == 'mysql':
		return '''insert into openk_semaforo.itens_pedido (pedido_id, sku, codigo_erp, quantidade, ean, valor, valor_desconto, valor_frete) 
					values (%s, %s, %s, %s, %s, %s, %s, %s) '''
	elif connection_type.lower() == 'oracle':
		return '''INSERT INTO OPENK_SEMAFORO.ITENS_PEDIDO (PEDIDO_ID, SKU, CODIGO_ERP, QUANTIDADE, EAN, VALOR, VALOR_DESCONTO, VALOR_FRETE)
					VALUES (:1, :2, :3, :4, :5, :6, :7, :8)'''
	elif connection_type.lower() == 'sqlserver':
		return '''insert into openk_semaforo.itens_pedido (pedido_id, sku, codigo_erp, quantidade, ean, valor, valor_desconto, valor_frete) 
					values (?, ?, ?, ?, ?, ?, ?, ?) '''


def get_query_client(connection_type: str):
	if connection_type.lower() == 'mysql':
		return 'select id from openk_semaforo.cliente where email = %s'
	elif connection_type.lower() == 'oracle':
		return 'SELECT ID FROM OPENK_SEMAFORO.CLIENTE WHERE EMAIL = :email'
	elif connection_type.lower() == 'sqlserver':
		return 'select id from openk_semaforo.cliente where email = ?'


def get_query_client_erp(connection_type: str):
	if connection_type.lower() == 'mysql':
		return 'select id from openk_semaforo.cliente where email = %s'
	elif connection_type.lower() == 'oracle':
		return 'SELECT ID FROM OPENK_SEMAFORO.CLIENTE WHERE CLIENTE_ERP = :cliente_erp '
	elif connection_type.lower() == 'sqlserver':
		return 'select id from openk_semaforo.cliente where email = ?'


def get_update_client_sql(connection_type: str):
	if connection_type.lower() == 'mysql':
		# TODO fazer query para dar update no cliente em mysql
		return '''update openk_semaforo.cliente
					SET nome = %s
					  , razao_social = %s
					  , cpf = %s
					  , cnpj = %s
					  , email = %s
					  , telefone_residencial = %s
					  , telefone_celular = %s
					  , cep = %s
					  , tipo_logradouro = %s
					  , logradouro = %s
					  , numero = %s
					  , complemento = %s
					  , bairro = %s
					  , cidade = %s
					  , estado = %s
					  , referencia = %s
					WHERE cliente_erp = %s '''
	elif connection_type.lower() == 'oracle':
		return '''UPDATE OPENK_SEMAFORO.CLIENTE
					SET NOME = :1
					  , RAZAO_SOCIAL = :2
					  , CPF = :3
					  , CNPJ = :4
					  , EMAIL = :5
					  , TELEFONE_RESIDENCIAL = :6
					  , TELEFONE_CELULAR = :7
					  , CEP = :8
					  , TIPO_LOGRADOURO = :9
					  , LOGRADOURO = :10
					  , NUMERO = :11
					  , COMPLEMENTO = :12
					  , BAIRRO = :13
					  , CIDADE = :14
					  , ESTADO = :15
					  , REFERENCIA = :16
					WHERE CLIENTE_ERP = :17 '''
	elif connection_type.lower() == 'sqlserver':
		return '''update openk_semaforo.cliente
							SET nome = ?
							  , razao_social = ?
							  , cpf = ?
							  , cnpj = ?
							  , email = ?
							  , telefone_residencial = ?
							  , telefone_celular = ?
							  , cep = ?
							  , tipo_logradouro = ?
							  , logradouro = ?
							  , numero = ?
							  , complemento = ?
							  , bairro = ?
							  , cidade = ?
							  , estado = ?
							  , referencia = ?
							WHERE cliente_erp = ? '''


def get_query_order(connection_type: str):
	if connection_type.lower() == 'mysql':
		return 'select id from openk_semaforo.pedido where pedido_id = %s and data_sincronizacao is null'
	elif connection_type.lower() == 'oracle':
		return 'SELECT ID FROM OPENK_SEMAFORO.PEDIDO WHERE PEDIDO_ID = :order_id AND DATA_SINCRONIZACAO IS NULL'
	elif connection_type.lower() == 'sqlserver':
		return 'select id from openk_semaforo.pedido where pedido_id = ? and data_sincronizacao is null'


def get_command_parameter(connection_type: str, parameters: list):
	if connection_type.lower() == 'mysql':
		return tuple(parameters)
	elif connection_type.lower() == 'oracle':
		return parameters
	elif connection_type.lower() == 'sqlserver':
		return parameters


def get_order_protocol_command(connection_type: str):
	if connection_type.lower() == 'mysql':
		return 'update openk_semaforo.pedido set data_sincronizacao = now(), codigo_referencia = %s where pedido_id = %s'
	elif connection_type.lower() == 'oracle':
		return 'UPDATE OPENK_SEMAFORO.PEDIDO SET DATA_SINCRONIZACAO = SYSDATE, CODIGO_REFERENCIA = :order_erp_id WHERE PEDIDO_ID = :order_id'
	elif connection_type.lower() == 'sqlserver':
		return 'update openk_semaforo.pedido set data_sincronizacao = getdate(), codigo_referencia = ? where pedido_id = ?'


def get_client_protocol_command(connection_type: str):
	# TODO atualizar esses comandos
	if connection_type.lower() == 'mysql':
		return 'update openk_semaforo.pedido set data_sincronizacao = now(), codigo_referencia = %s where pedido_id = %s'
	elif connection_type.lower() == 'oracle':
		return 'UPDATE OPENK_SEMAFORO.CLIENTE SET CLIENTE_ERP = :client_erp_id WHERE PEDIDO_ID = :order_id'
	elif connection_type.lower() == 'sqlserver':
		return 'update openk_semaforo.cliente set data_sincronizacao = getdate(), codigo_referencia = ? where pedido_id = ?'



import logging
import src.database.connection as database
from src.database import queries
from src.database.utils import DatabaseConfig
import src.database.utils as utils
from src.entities.product import Product
import src.api.okvendas as api_okvendas

logger = logging.getLogger()


def job_update_products_semaphore(job_config_dict: dict):
    try:
        db_config = utils.get_database_config(job_config_dict)
        if db_config.sql is None:
            logger.warning(job_config_dict.get('job_name') + ' | Comando sql para criar produtos nao encontrado')
            return

        db = database.Connection(db_config)
        connection = db.get_conect()
        cursor = connection.cursor()

        cursor.execute(db_config.sql)
        cursor.close()

        logger.info(job_config_dict.get('job_name') + f' | Produtos marcados para atualizar no banco semaforo: {cursor.rowcount}')
        connection.commit()
        connection.close()

    except Exception as e:
        logger.error(job_config_dict.get('job_name') + f' | Erro ao atualizar produtos no banco semaforo: {e}', exc_info=True)


def job_insert_products_semaphore(job_config_dict: dict):
    """
    Job que realiza a insercao dos produtos no banco semaforo
    Args:
        job_config_dict: Dicionario contendo configuracoes do job (obtidos na api oking)
    """
    try:
        db_config = utils.get_database_config(job_config_dict)
        if db_config.sql is None:
            logger.warning(job_config_dict.get('job_name') + ' | Comando sql para criar produtos nao encontrado')
            return

        # abre a connection com o banco
        db = database.Connection(db_config)
        connection = db.get_conect()
        cursor = connection.cursor()
        logger.info(job_config_dict.get('job_name') + f' | Conectado ao banco de dados {db_config.db_name}')

        try:
            insert_list = [insert for insert in db_config.sql.split(';') if insert != '']
            # executa o insert
            for i in insert_list:
                logger.info(job_config_dict.get('job_name') + f' | Executando comando sql {i}')
                cursor.execute(i)

            cursor.close()
            connection.commit()
            connection.close()
        except Exception as e:
            logger.error(job_config_dict.get('job_name') + f' | Erro ao inserir produtos no banco semaforo: {e}', exc_info=True)
            cursor.close()
            connection.close()

    except Exception as e:
        logger.error(job_config_dict.get('job_name') + f' | Erro ao durante a execucao do job: {e}', exc_info=True)


def job_send_products(job_config_dict: dict):
    """
    Job que realiza a leitura dos produtos contidos no banco semaforo e envia para a api okvendas
    Args:
        job_config_dict: Dicionario contendo configuracoes do job (obtidos na api oking)
    """
    try:
        db_config = utils.get_database_config(job_config_dict)
        if db_config.sql is None:
            logger.warning(job_config_dict.get('job_name') + ' | Comando sql para criar produtos nao encontrado')
            return

        produtos = query_products(db_config)
        if len(produtos) <= 0:
            logger.warning(job_config_dict.get('job_name') + ' | Nao existem produtos para criar no momento')
            return

        produtos_protocolar = []
        for prod in produtos:
            try:
                response = api_okvendas.post_produtos([prod])

                for res in response:
                    if res.status > 1:
                        logger.warning(job_config_dict.get('job_name') + f' | Erro ao gerar produto {res.codigo_erp} na api okvendas. Erro gerado na api: {res.message}')
                    else:
                        logger.info(job_config_dict.get('job_name') + f' | Produto {res.codigo_erp} criado com sucesso')
                        produtos_protocolar.append({'codigo_erp': prod.codigo_erp, 'codigo_erp_sku': prod.preco_estoque[0].codigo_erp_atributo})
            except Exception as e:
                logger.error(job_config_dict.get('job_name') + f' | Erro durante o envio de produtos: {e}', exc_info=True)

        protocol_products(produtos_protocolar, db_config)

    except Exception as e:
        logger.error(job_config_dict.get('job_name') + f' | Erro ao durante a execucao do job: {e}', exc_info=True)


def query_products(db_config: DatabaseConfig):
    """
    Consulta os produtos contidos no banco semaforo juntamente com os dados do banco do ERP
    Args:
        db_config: Configuracao do banco de dados

    Returns:
        Lista de produtos
    """
    # abre a connection com o banco
    db = database.Connection(db_config)
    connection = db.get_conect()
    # connection.start_transaction()
    cursor = connection.cursor()

    # obtem os dados do banco
    # logger.warning(query)
    cursor.execute(db_config.sql)
    columns = [col[0] for col in cursor.description]
    results = [dict(zip(columns, row)) for row in cursor.fetchall()]

    cursor.close()
    connection.close()

    produtos = []
    for result in results:
        produtos.append(Product(result))

    return produtos


def protocol_products(products: list, db_config: DatabaseConfig) -> None:
    """
    Protocola no banco semaforo os produtos que foram enviados para a api okvendas
    Args:
        products: Lista de produtos enviados para a api okvendas
        db_config: Configuracao do banco de dados
    """
    try:
        if len(products) > 0:
            db = database.Connection(db_config)
            connection = db.get_conect()
            cursor = connection.cursor()
            for prod in products:
                try:
                    dados_produto = [prod['codigo_erp'], prod['codigo_erp_sku']]
                    logger.info(f'Protocolando sku {dados_produto[1]}')
                    cursor.execute(queries.get_product_protocol_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, dados_produto))
                    logger.info(f'Linhas afetadas {cursor.rowcount}')
                except Exception as e:
                    logger.error(f'Erro ao protocolar sku {prod["codigo_erp_sku"]}: {e}', exc_info=True)
            cursor.close()
            connection.commit()
            connection.close()
    except Exception as e:
        raise e



class Cliente:

    def __init__(self, codigo_referencia, nome, razao_social,
                 cpf, rg, data_nascimento, cnpj, sexo,
                 email, orgao, RegistroEstadual, TelefoneResidencial,
                 TelefoneCelular, Endereco, EnderecoEntrega):
        self.codigo = codigo_referencia
        self.nome = nome
        self.razao_social = razao_social
        self.cpf = cpf
        self.rg = rg
        self.data_nascimento = data_nascimento
        self.cnpj = cnpj
        self.sexo = sexo
        self.email = email
        self.orgao = orgao
        self.RegistroEstadual = RegistroEstadual
        self.TelefoneResidencial = TelefoneResidencial
        self.TelefoneCelular = TelefoneCelular

        if Endereco is not None:
            self.Endereco = EnderecoCliente(**Endereco)

        if EnderecoEntrega is not None:
            self.EnderecoEntrega = EnderecoCliente(**EnderecoEntrega)


class EnderecoCliente:

    def __init__(self, cep, logradouro, numero,
                 complemento, bairro, cidade, estado,
                 pais, referencia, descricao, tipo_logradouro):
        self.cep = cep
        self.logradouro = logradouro
        self.numero = numero
        self.complemento = complemento
        self.bairro = bairro
        self.cidade = cidade
        self.estado = estado
        self.pais = pais
        self.referencia = referencia
        self.descricao = descricao
        self.tipo_logradouro = tipo_logradouro

    def get_sql(self):
        sql = '''INSERT INTO lojasinfinity_openkuget.usuario
                        (id, loja_id, client_id, processado,
                        datapedido, valor, valor_frete,
                        status_canal_id, status, canal_venda_id,
                        data_criacao, forma_envio)
                VALUES (null, 1, 147, 1, '''
        + self.cep + ','
        + self.logradouro + ','
        + self.numero + ','
        + self.complemento + ','
        + self.bairro + ','
        + self.cidade + ','
        + self.estado + ','
        + self.tipo_logradouro
        + ')'
        return sql



import logging
import src.database.connection as database
import src.database.utils as utils
from src.database.utils import DatabaseConfig
import src.api.okvendas as api_okvendas
from src.entities.price import Price
from src.entities.response import PriceResponse
from src.database import queries
import src

logger = logging.getLogger()


def job_insert_prices_semaphore(job_config_dict: dict):
    """
    Job para preços no banco semáforo
    Args:
        job_config_dict: Configuração do job
    """
    db_config = utils.get_database_config(job_config_dict)
    if db_config.sql is None:
        logger.warning(job_config_dict.get('job_name') + ' | Comando sql para inserir precos no semaforo nao encontrado')
    else:
        db = database.Connection(db_config)
        conn = db.get_conect()
        cursor = conn.cursor()

        try:
            logger.info(job_config_dict.get('job_name') + ' | Inserindo precos no banco semaforo')
            logger.info(db_config.sql)
            cursor.execute(db_config.sql)
            logger.info(job_config_dict.get('job_name') + f' | {cursor.rowcount} precos inseridos no banco semaforo')
        except Exception as ex:
            logger.error(job_config_dict.get('job_name') + ' | Erro ' + str(ex), exc_info=True)

        cursor.close()
        conn.commit()
        conn.close()


def job_send_prices(job_config_dict: dict):
    """
    Job para realizar a atualização de preços
    Args:
        job_config_dict: Configuração do job
    """
    try:
        db_config = utils.get_database_config(job_config_dict)
        if db_config.sql is None:
            logger.warning(job_config_dict.get('job_name') + ' | Comando sql para inserir precos no semaforo nao encontrado')
        else:
            prices_to_protocol = []
            prices = query_prices(db_config)
            if len(prices) > 0:
                logger.info(job_config_dict.get('job_name') + f' | Produtos para atualizar preco {len(prices)}')

                for price in prices:
                    try:
                        response = api_okvendas.post_prices(src.client_data.get('url_api') + '/catalogo/preco', price, src.client_data.get('token_api'))
                        if response.status == 1:
                            prices_to_protocol.append(response.codigo_erp)
                        else:
                            logger.warning(job_config_dict.get('job_name') + f' | Nao foi possivel atualizar o preco do sku {response.codigo_erp}. Erro recebido da Api: {response.message}')
                    except Exception as e:
                        logger.warning(job_config_dict.get('job_name') + f' | Falha ao atualizar preco do sku {price.codigo_erp}: {str(e)}')

                protocol_prices(db_config, prices_to_protocol)

            else:
                logger.warning(job_config_dict.get('job_name') + ' | Nao existem precos a serem enviados no momento')
    except Exception as e:
        logger.error(job_config_dict.get('job_name') + f' | Erro durante execucao do job: {e}')


def query_prices(db_config: DatabaseConfig) -> list[Price]:
    """
    Consulta os precos para atualizar no banco de dados
    Args:
        db_config: Configuracao do banco de dados

    Returns:
        Lista de preços para atualizar
    """
    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    prices = list[PriceResponse]
    try:
        cursor.execute(db_config.sql)
        rows = cursor.fetchall()
        columns = [col[0] for col in cursor.description]
        results = [dict(zip(columns, row)) for row in rows]
        cursor.close()
        conn.close()
        if len(results) > 0:
            prices = [Price(**p) for p in results]

    except Exception as ex:
        logger.error(f' Erro ao consultar precos no banco semaforo:  {ex}')

    return prices


def protocol_prices(db_config: DatabaseConfig, prices: list[str]) -> None:
    """
    Protocola os preços no banco semáforo
    Args:
        db_config: Configuração do banco de dados
        prices: Lista de skus para protocolar
    """
    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    sql = queries.get_price_protocol_command(db_config.db_type)
    for p in prices:
        try:
            cursor.execute(sql, queries.get_command_parameter(db_config.db_type, [p]))
        except Exception as e:
            logger.warning(f'Erro ao protocolar preco do sku {p}: {e}')

    cursor.close()
    conn.commit()
    conn.close()




class Tracking:
    def __init__(self, pedido_id: int, codigo_rastreio: str, link_rastreio: str, data_previsao_entrega: str, codigo_erp_rastreio: str, codigo_sku: str,
                 cnpj_transportadora: str, transportadora: str, codigo_transportadora: str, tipo_servico: str, quantidade: int):
        self.id: int = pedido_id
        self.codigoRastreio: str = codigo_rastreio
        self.linkRastreio: str = link_rastreio
        self.data_previsao_entrega: str = data_previsao_entrega
        self.observacao: str = 'Pedido atualizado via Oking - Api Okvendas'
        self.codigo_erp: str = codigo_erp_rastreio
        self.codigo_sku: str = codigo_sku
        self.cnpj_transportadora: str = cnpj_transportadora
        self.transportadora: str = transportadora
        self.codigo_transportadora: str = codigo_transportadora
        self.tipo_servico: str = tipo_servico
        self.quantidade: int = quantidade



import requests as req
import jsonpickle
import logging

logger = logging.getLogger()


class Module:
    def __init__(self, job: str, comando_sql: str, unidade_tempo: str, tempo_execucao: int):
        self.job_name = job
        self.sql = comando_sql
        self.time_unit = unidade_tempo
        self.time = tempo_execucao


def get(url: str, params: dict = None):
    response = req.get(url, params)
    if str(response.status_code).startswith('2'):
        return jsonpickle.decode(response.content)
    else:
        logger.error(f'Erro ao executar GET: {url} | Code: {response.status_code} | {response.content}')

    return None



from typing import List
from datetime import datetime
from src.entities.product import Product
from src.entities.product import ItemBrinde
from src.entities.product import ItemPersonalizado
from src.entities.cliente import Cliente
from src.entities.pagamento import Pagamento


class Pedido:

    def __init__(self, id, pedido_venda_id, data_pedido, data_geracao,
                 codigo_referencia, valor_total, valor_forma_pagamento,
                 valor_desconto, valor_frete, status: str, quantidade_titulos,
                 previsao_entrega, codigo_rastreio, canal_id,
                 transportadora_id, transportadora,
                 servico_id, servico, codigo_carga,
                 transacao, usuario, pagamento: list[Pagamento],
                 itens, itens_brinde, itens_personalizados,
                 forma_pagamento_parceiro, forma_envio_parceiro,
                 pedido_nota_fiscal, protocolo, valor_adicional_forma_pagamento):
        self.id = id
        self.pedido_venda_id = pedido_venda_id
        self.data_pedido = data_pedido
        self.data_geracao = data_geracao
        self.codigo_referencia = codigo_referencia
        self.valor_total = valor_total
        self.valor_adicional_forma_pagamento = valor_adicional_forma_pagamento
        self.valor_forma_pagamento = valor_forma_pagamento
        self.valor_desconto = valor_desconto
        self.valor_frete = valor_frete
        self.status = status
        self.quantidade_titulos = quantidade_titulos
        self.previsao_entrega = previsao_entrega
        self.codigo_rastreio = codigo_rastreio
        self.canal_id = canal_id
        self.transportadora_id = transportadora_id
        self.transportadora = transportadora
        self.servico_id = servico_id
        self.servico = servico
        self.codigo_carga = codigo_carga
        self.transacao = transacao  # {Transacao}
        # {FormaPagamento}
        self.forma_pagamento_parceiro = forma_pagamento_parceiro
        self.forma_envio_parceiro = forma_envio_parceiro
        self.pedido_nota_fiscal = pedido_nota_fiscal  # [NotaFiscal]
        self.protocolo = protocolo
        self.novo_cliente = True

        if usuario is not None:
            self.usuario = Cliente(**usuario)

        self.pagamento: List[Pagamento]
        self.pagamento = []
        if pagamento is not None:
            for pag in pagamento:
                p = Pagamento(**pag)
                self.pagamento.append(p)

        self.itens: List[Product]
        self.itens = []
        if itens is not None:
            for item in itens:
                self.itens.append(OrderItem(**item))

        self.itens_brinde: List[ItemBrinde]
        self.itens_brinde = []
        if itens_brinde is not None:
            for item in itens_brinde:
                p = ItemBrinde(**item)
                self.itens_brinde.append(p)

        self.itens_personalizados: List[ItemPersonalizado]
        self.itens_personalizados = []
        if itens_personalizados is not None:
            for item in itens_personalizados:
                p = ItemPersonalizado(**item)
                self.itens_personalizados.append(p)

    def get_insert(self):
        sql = '''INSERT INTO lojasinfinity_openkuget.pedido_erp
                        (id, loja_id, client_id, processado,
                        datapedido, valor, valor_frete,
                        status_canal_id, status, canal_venda_id,
                        data_criacao, forma_envio)
                VALUES (null, 1, 147, 1, '''
        + self.data_pedido + ','
        + self.valor_total + ','
        + self.valor_frete + ','
        + self.status + ','
        + self.status + ','
        + self.canal_id + ','
        + self.data_geracao + ','
        + self.forma_envio_parceiro
        + ')'
        return sql

    def get_dados_cliente(self):
        # tes = stT.ljust(40, "N")
        # tes = stT.rjust(15, "0")
        tipo_endereco = 'RUA'
        documento = self.usuario.cpf if self.usuario.cpf is not None \
            else self.usuario.cnpj

        nome_cliente = self.usuario.nome
        if self.usuario.nome is None:
            nome_cliente = self.usuario.razao_social

        dados = '1;C;' + \
            documento.rjust(15, "0") + ';' + \
            nome_cliente.ljust(40, " ") + ';' + \
            tipo_endereco + ';' + \
            self.usuario.Endereco.logradouro.ljust(40, " ") + ';' + \
            self.usuario.Endereco.numero.ljust(5, " ") + ';' + \
            self.usuario.Endereco.cep + ';' + \
            self.usuario.Endereco.bairro.ljust(20, " ") + ';' + \
            self.usuario.email.ljust(40, " ") + ';' + \
            self.usuario.TelefoneCelular + ';' + \
            documento.rjust(15, "0") + ';\n'
        return dados

    def get_dados_pedido(self):
        forma_pagamento = ""
        if len(self.pagamento) > 0:
            forma_pagamento = self.pagamento[0].opcao_pagamento

        dt_pedido = ""
        if self.data_pedido is not None:
            data = datetime.strptime(self.data_pedido, "%Y-%m-%dT%H:%M:%S")
            dt_pedido = data.strftime("%d/%m/%Y")

        documento = self.usuario.cpf if self.usuario.cpf is not None \
            else self.usuario.cnpj

        nome_cliente = self.usuario.nome
        if self.usuario.nome is None:
            nome_cliente = self.usuario.razao_social

        dados = '1;P;' + \
            str(self.id).rjust(12, "0") + ';' + \
            documento.rjust(15, "0") + ';' + \
            nome_cliente.ljust(15, " ") + ';' + \
            dt_pedido + ';' + \
            str(self.valor_total).rjust(15, "0") + ';' + \
            str(self.valor_desconto).rjust(15, "0") + ';' + \
            forma_pagamento.ljust(100, " ") + ';\n'

        return dados

    def get_dados_lista_itens(self):
        d_itens = ''
        for it in self.itens:
            d_itens = d_itens + '1;I;' + \
                it.sku_principal.rjust(12, "0") + ';' + \
                str(it.valor_desconto).rjust(15, "0") + ';' + \
                str(it.value).rjust(15, "0") + ';' + \
                str(it.quantidade).rjust(15, "0") + ';' + \
                it.ean.ljust(13, " ") + ';' + \
                str(self.valor_frete).rjust(12, "0") + ';\n'

        return d_itens

    def get_insert_itens(self):
        sql = '''INSERT INTO lojasinfinity_openkuget.item_pedido_erp'''
        return sql

    def get_comando_proc(self):
        # STP_PROCESSA_PEDIDO3(P_TIPREG VARCHAR2,P_TEXTO VARCHAR2)
        sql = "call STP_PROCESSA_PEDIDO3 (" + \
            str(self.id) + "," + \
            " conteudo de ARQUIVO )"
        return sql

    def get_insert_cliente(self):
        sql = '''INSERT INTO lojasinfinity_openkuget.cliente
                        (id, loja_id, client_id, processado,
                        datapedido, valor, valor_frete,
                        status_canal_id, status, canal_venda_id,
                        data_criacao, forma_envio)
                VALUES (null, 1, 147, 1, '''
        + self.data_pedido + ','
        + self.valor_total + ','
        + self.valor_frete + ','
        + self.status + ','
        + self.status + ','
        + self.canal_id + ','
        + self.data_geracao + ','
        + self.forma_envio_parceiro
        + ')'
        return sql

    def get_insert_endereco(self):
        sql = '''INSERT INTO lojasinfinity_openkuget.endereco_cliente
                        (id, loja_id, client_id, processado,
                        datapedido, valor, valor_frete,
                        status_canal_id, status, canal_venda_id,
                        data_criacao, forma_envio)
                VALUES (null, 1, 147, 1, '''
        + self.data_pedido + ','
        + self.valor_total + ','
        + self.valor_frete + ','
        + self.status + ','
        + self.status + ','
        + self.canal_id + ','
        + self.data_geracao + ','
        + self.forma_envio_parceiro
        + ')'
        return sql





class Transacao:

    def __init__(self, transacao_id, numero_autorizacao,
                 nsu, mensagem_retorno):
        self.transacao_id = transacao_id
        self.numero_autorizacao = numero_autorizacao
        self.nsu = nsu
        self.mensagem_retorno = mensagem_retorno


class FormaEnvio:

    def __init__(self, codigo_rastreio, forma_envio,
                 tipo_envio, status_envio, data_previsao_postagem,
                 modo_envio, plp, rota, mega_rota):
        self.codigo_rastreio = codigo_rastreio
        self.forma_envio = forma_envio
        self.tipo_envio = tipo_envio
        self.status_envio = status_envio
        self.data_previsao_postagem = data_previsao_postagem
        self.modo_envio = modo_envio
        self.plp = plp
        self.rota = rota
        self.mega_rota = mega_rota


class NotaFiscal:

    def __init__(self, nota_fiscal_id, pedido_id,
                 num_fiscal, num_serie, access_key,
                 valor_total_nota, data_emissao, dt_cadastro,
                 xml, link_danfe, link_xml, quantidade_volume):
        self.nota_fiscal_id = nota_fiscal_id
        self.pedido_id = pedido_id
        self.num_fiscal = num_fiscal
        self.num_serie = num_serie
        self.access_key = access_key
        self.valor_total_nota = valor_total_nota
        self.data_emissao = data_emissao
        self.dt_cadastro = dt_cadastro
        self.xml = xml
        self.link_danfe = link_danfe
        self.link_xml = link_xml
        self.quantidade_volume = quantidade_volume


class Fila:
    def __init__(self, pedido_id, data_pedido,
                 status, protocolo, data_fila,
                 observacao, valor_total):
        self.pedido_id = pedido_id
        self.data_pedido = data_pedido
        self.status = status
        self.protocolo = protocolo
        self.data_fila = data_fila
        self.observacao = observacao
        self.valor_total = valor_total


def toListFilaDecoder(fila):
    lista = []
    for item in fila:
        lista.append(Fila(**item))

    return lista


# Serializing
# data = json.dumps(pedido, default=lambda o: o.__dict__,
#                   sort_keys=True, indent=4)
# Deserializing
# decoded_data = Pedido.from_json(json.loads(data))




class Price:
    def __init__(self, DATA_SINCRONIZACAO, DATA_ATUALIZACAO, CODIGO_ERP, PRECO_ATUAL, PRECO_CUSTO, PRECO_LISTA, CODIGO_EXTERNO_CAMPANHA):
        self.codigo_erp = CODIGO_ERP
        self.preco_atual = PRECO_ATUAL
        self.preco_lista = PRECO_LISTA
        self.preco_custo = PRECO_CUSTO
        self.parceiro = 1
        self.codigo_externo_campanha = CODIGO_EXTERNO_CAMPANHA


class Queue:
    def __init__(self, pedido_id, data_pedido, status, protocolo, data_fila, observacao, valor_total, numero_pedido_externo):
        self.order_id = pedido_id
        self.date = data_pedido
        self.status = status
        self.protocol = protocolo


class Order:
    def __init__(self, id: int, pedido_venda_id: str, data_pedido: str, data_geracao: str, codigo_referencia: str,
                 valor_total: float, valor_forma_pagamento: float, valor_desconto: float, valor_frete: float,
                 status: str, quantidade_titulos: int, previsao_entrega: str, codigo_rastreio: str,
                 canal_id: int, transportadora_id: str, transportadora: str, servico_id: int, servico: str,
                 codigo_carga: str, canal_site: str, valor_adicional_forma_pagamento: float, codigo_pedido_canal_alternativo: str,
                 protocolo: str, transacao, usuario: dict, pagamento: list, itens: list,
                 itens_brinde, itens_personalizados, forma_pagamento_parceiro, forma_envio_parceiro: dict,
                 pedido_nota_fiscal):
        self.order_id: int = id
        self.order_code: str = pedido_venda_id
        self.date: str = data_pedido
        self.erp_code: str = codigo_referencia
        self.total_amount: float = valor_total
        self.total_discount: float = valor_desconto
        self.freight_amount: float = valor_frete
        self.status: str = status
        self.delivery_forecast = previsao_entrega
        self.tracking_code = codigo_rastreio
        self.carrier: str = transportadora
        self.user: User = User(**usuario)
        self.additional_payment_amount: float = valor_adicional_forma_pagamento
        self.items: list[OrderItem] = [OrderItem(**i) for i in itens]

        payments: list[Payment] = [Payment(**p) for p in pagamento]
        if len(payments) > 0:
            self.paid_date = payments[0].paid_date
            self.flag = payments[0].flag
            self.erp_payment_condition = payments[0].erp_payment_condition
            self.parcels = payments[0].parcels
            self.payment_type = payments[0].type

        partner_shipping_methods: list[PartnerShippingMethod] = [PartnerShippingMethod(**forma_envio_parceiro)]
        if len(partner_shipping_methods) > 0:
            self.shipping_mode = partner_shipping_methods[0].shipping_mode


class PartnerShippingMethod:
    def __init__(self, codigo_rastreio: str, forma_envio: str, tipo_envio: str, status_envio: str, data_previsao_postagem: str, modo_envio: str,
                 plp: str, rota: str, mega_rota: str):
        self.shipping_mode: str = modo_envio


class Payment:
    def __init__(self, opcao_pagamento: str, parcelas: int, bandeira: str, condicao_pagamento_erp: str, tabela_financiamento_rsvarejo: str,
                 tipo_venda_rsvarejo: str, canal_venda_id: str, canal_venda: str, numero_cupom: str, valor_cupom: float, codigo_compra: str,
                 codigo_pedido_canal: str, data_movimento: str, codigo_mercado: str, titulos):
        self.type: str = opcao_pagamento
        self.erp_payment_condition: str = condicao_pagamento_erp
        self.parcels: int = parcelas
        self.flag: str = bandeira
        self.paid_date: str = titulos[0]['data_pago']


class User:
    def __init__(self, codigo_referencia: str, nome: str, razao_social: str, cpf: str, rg: str, data_nascimento: str, cnpj: str,
                 sexo: str, email: str, orgao: str, RegistroEstadual: str, TelefoneResidencial: str, TelefoneCelular: str,
                 Endereco: dict, EnderecoEntrega: dict):
        self.erp_code: str = codigo_referencia
        self.name: str = nome
        self.company_name: str = razao_social
        self.cpf: str = cpf
        self.cnpj: str = cnpj
        self.email: str = email
        self.residential_phone: str = TelefoneResidencial
        self.mobile_phone: str = TelefoneCelular
        self.address: Address = Address(**EnderecoEntrega)
        self.cnpj: str = cnpj


class Address:
    def __init__(self, cep: str, logradouro: str, numero: str, complemento: str, bairro: str, cidade: str,
                 estado: str, pais: str, referencia: str, descricao: str, tipo_logradouro: str):
        self.zipcode: str = cep
        self.address_line: str = logradouro
        self.number: str = numero
        self.complement: str = complemento
        self.neighbourhood: str = bairro
        self.city: str = cidade
        self.state: str = estado
        self.reference: str = referencia
        self.address_type: str = tipo_logradouro or 'Rua'


class OrderItem:
    def __init__(self, sku_principal: str, sku_variacao: str, sku_reference: str, hierarquia_variacao: str, is_restock: bool, codigo_externo_restock: str,
                 ean: str, quantidade: int, value: float, valor_desconto: float, altura: float, comprimento: float, largura: float, peso: float, volume: float,
                 filial_expedicao: str, filial_faturamento: str, cnpj_filial_venda: str, valor_frete: float, percentual_comissao: float, valor_comissao: float,
                 valor_comissao_frete: float):
        self.sku: str = sku_reference
        self.erp_code: str = sku_variacao
        self.quantity: int = quantidade
        self.ean: str = ean
        self.value: float = value
        self.discount: float = valor_desconto
        self.freight_value: float = valor_frete




class DatabaseConfig:
    def __init__(self, sql: str, db_type: str, db_name: str, db_host: str, db_user: str, db_pwd: str, db_client: str):
        self.db_type = db_type
        self.sql = sql
        self.db_name = db_name
        self.db_host = db_host
        self.db_user = db_user
        self.db_pwd = db_pwd
        self.db_client = db_client


def get_database_config(job_config: dict):
    return DatabaseConfig(
        job_config.get('sql'),
        job_config.get('db_type'),
        job_config.get('db_name'),
        job_config.get('db_host'),
        job_config.get('db_user'),
        job_config.get('db_pwd'),
        job_config.get('db_client'))


from setuptools import setup, find_packages
from os import path


def read(fname):
    return open(path.join(path.dirname(__file__), fname)).read()


def save_version(v: str):
    f = open('version.txt', 'w')
    f.write(f'\tOking {v} - Openk Tecnologia')
    f.close()


version = '0.6.8'
save_version(version)
setup(
    name="oking",
    version=version,
    author="Openk Tecnologia",
    author_email="<suporte.b2c@openk.com.br>",
    description='Pacote de integração de produtos, preço, estoque e pedidos com o sistema OkVendas da Openk',
    long_description_content_type="text/markdown",
    long_description=read('README.md'),
    packages=find_packages(include=['src', 'src.*']),
    install_requires=['mysql-connector-python',
                      'schedule',
                      'requests',
                      'configparser',
                      'logger',
                      'cx-Oracle',
                      'jsonpickle',
                      'pyodbc'],
    keywords=['python', 'oking', 'openk', 'okvendas', 'ok'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ],
    entry_points={
        'console_scripts': ['oking=src.__main__:main']
    }
)



import logging
from datetime import datetime
from time import sleep

import src
import src.api.okvendas as api_okvendas
import src.database.connection as database
import src.database.utils as utils
from src.database import queries
from src.database.utils import DatabaseConfig
from src.entities.tracking import Tracking
from src.entities.invoice import Invoice
from src.entities.order import Order

logger = logging.getLogger()

default_limit = 50
queue_status = {
    'pending': 'PEDIDO',
    'paid': 'PEDIDO_PAGO',
    'shipped': 'ENCAMINHADO_ENTREGA',
    'delivered': 'ENTREGUE',
    'canceled': 'CANCELADO',
    'no_invoice': 'SEM_NOTA_FISCAL',
    'invoiced': 'FATURADO'
}


def define_job_start(job_config: dict) -> None:
    global current_job
    current_job = job_config.get('job_name')
    if current_job == 'internaliza_pedidos_job':  # Inicia o job a partir dos pedidos AgPagamento
        job_orders(job_config, True)
    else:  # Inicia o job a partir dos pedidos Confirmados
        job_orders(job_config)


def job_orders(job_config: dict, start_at_pending: bool = False) -> None:
    try:
        db_config = utils.get_database_config(job_config)
        if start_at_pending:
            process_order_queue(queue_status.get('pending'), db_config)

        process_order_queue(queue_status.get('paid'), db_config)

        process_order_queue(queue_status.get('canceled'), db_config)

        # process_order_queue(queue_status.get('invoiced'), db_config)

        process_order_queue(queue_status.get('shipped'), db_config)

        process_order_queue(queue_status.get('delivered'), db_config)

    except Exception as e:
        logger.error(current_job + f' | Erro ao inicializar job: {str(e)}')


def job_invoice_orders(job_config: dict):
    try:
        db_config = utils.get_database_config(job_config)
        if db_config.sql is None:
            logger.warning(job_config.get('job_name') + ' | Comando sql para baixar notas fiscais nao encontrado')
        else:
            invoices = query_invoices(db_config)
            for invoice in invoices:
                try:
                    invoice_sent = api_okvendas.post_invoices(src.client_data.get('url_api') + '/pedido/faturar', invoice, src.client_data.get('token_api'))
                    if invoice_sent is None:
                        logger.info(job_config.get('job_name') + f' | NF do pedido {invoice.id} enviada com sucesso para api okvendas')
                        continue
                    else:
                        logger.error(job_config.get('job_name') + f' | Falha ao enviar NF do pedido {invoice.id} para api okvendas: {invoice_sent.message}')
                except Exception as e:
                    logger.error(job_config.get('job_name') + f' | Falha ao enviar NF do pedido {invoice.id}: {str(e)}')
    except Exception as e:
        logger.error(job_config.get('job_name') + f' | Falha na execução do job: {str(e)}')


def job_send_erp_tracking(job_config: dict):
    try:
        db_config = utils.get_database_config(job_config)
        if db_config.sql is None:
            logger.warning(job_config.get('job_name') + ' | Comando sql para consultar rastreios nao encontrado')
        else:
            trackings = query_trackings(db_config)
            for tracking in trackings:
                try:
                    tracking_sent = api_okvendas.post_tracking(src.client_data.get('url_api') + '/pedido/encaminhar', tracking, src.client_data.get('token_api'))
                    if tracking_sent is None:
                        logger.info(job_config.get('job_name') + f' | Rastreio do pedido {tracking.id} enviada com sucesso para api okvendas')
                        continue
                    else:
                        logger.error(job_config.get('job_name') + f' | Falha ao enviar rastreio do pedido {tracking.id} para api okvendas: {tracking_sent.message}')
                except Exception as e:
                    logger.error(job_config.get('job_name') + f' | Falha ao enviar rastreio do pedido {tracking.id}: {str(e)}')
    except Exception as e:
        logger.error(job_config.get('job_name') + f' | Falha na execução do job: {str(e)}')


def process_order_queue(status: str, db_config: DatabaseConfig) -> None:
    try:
        queue = api_okvendas.get_order_queue(
            url=src.client_data.get('url_api') + '/pedido/fila/{0}',
            token=src.client_data.get('token_api'),
            status=status,
            limit=default_limit)

        qty = 0
        for q_order in queue:
            try:
                sleep(0.5)
                qty = qty + 1
                logger.info(current_job + f' | Iniciando processamento ({qty} de {len(queue)}) pedido {q_order.order_id}')
                order = api_okvendas.get_order(
                    url=src.client_data.get('url_api') + '/pedido/{0}',
                    token=src.client_data.get('token_api'),
                    order_id=q_order.order_id)

                if order.erp_code is not None and order.erp_code != '':
                    if check_order_existence(db_config, order.order_id):
                        logger.info(current_job + ' | Pedido ja integrado com o ERP, chamando procedure de atualizacao...')
                        if call_update_order_procedure(db_config, order):
                            logger.info(current_job + ' | Pedido atualizado com sucesso')
                            protocol_order(db_config, order)
                    else:
                        logger.warning(current_job + f' | Pedido {order.order_id} nao existe no banco semaforo porem ja foi integrado previamente. Protocolando pedido...')
                        protocol_non_existent_order(order)

                else:
                    if check_order_existence(db_config, order.order_id):
                        logger.info(current_job + ' | Pedido já existente no banco semáforo, porem nao integrado com erp. Chamando procedures')
                        sp_success, client_erp_id, order_erp_id = call_order_procedures(db_config, q_order.order_id)
                        if sp_success:
                            logger.info(current_job + ' | Chamadas das procedures executadas com sucesso, protocolando pedido...')
                            protocol_order(db_config, order, order_erp_id, client_erp_id)
                    else:
                        logger.info(current_job + ' | Inserindo novo pedido no banco semaforo')
                        inserted = insert_temp_order(order, db_config)
                        if inserted:
                            logger.info(current_job + ' | Pedido inserido com sucesso, chamando procedures...')
                            sp_success, client_erp_id, order_erp_id = call_order_procedures(db_config, q_order.order_id)
                            if sp_success:
                                logger.info(current_job + ' | Chamadas das procedures executadas com sucesso, protocolando pedido...')
                                protocol_order(db_config, order, order_erp_id, client_erp_id)

            except Exception as e:
                logger.error(current_job + f' | Erro no processamento do pedido {q_order.order_id}: {str(e)}')
    except Exception as e:
        logger.error(current_job + f' | Erro ao inicializar job de processamento de pedidos: {str(e)}')


def protocol_order(db_config: DatabaseConfig, order: Order, order_erp_id: str = '', client_erp_id: str = '') -> None:
    db = database.Connection(db_config)
    try:
        if order_erp_id != '':
            updated_order_code = api_okvendas.put_order_erp_code(src.client_data.get('url_api') + '/pedido/integradoERP',
                                                                 src.client_data.get('token_api'),
                                                                 order.order_id,
                                                                 order_erp_id)
            if updated_order_code:
                logger.info(current_job + ' | Codigo Erp do pedido atualizado via api OkVendas')
            else:
                logger.warning(current_job + ' | Falha ao atualizar o Codigo Erp do pedido via api OkVendas')

        if client_erp_id != '':
            updated_client_code = api_okvendas.put_client_erp_code(src.client_data.get('url_api') + '/cliente/codigo',
                                                                   src.client_data.get('token_api'),
                                                                   {
                                                                       'cpf_cnpj': order.user.cpf if order.user.cpf is not None or order.user.cpf != '' else order.user.cnpj,
                                                                       'codigo_cliente': client_erp_id
                                                                   })
            if updated_client_code:
                logger.info(current_job + ' | Codigo Erp do cliente atualizado via api OkVendas')
            else:
                logger.warning(current_job + ' | Falha ao atualizar o Codigo Erp do cliente via api OkVendas')

        protocoled_order = api_okvendas.put_protocol_orders(order.order_id)
        if protocoled_order:
            logger.info(current_job + ' | Pedido protocolado via api OkVendas')
        else:
            logger.warning(current_job + ' | Falha ao protocolar pedido via api OkVendas')

        logger.info(current_job + ' | Protocolando pedido no banco semaforo')
        conn = db.get_conect()
        cursor = conn.cursor()
        cursor.execute(queries.get_order_protocol_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [order_erp_id, order.order_id]))
        cursor.close()
        conn.commit()
        conn.close()
    except Exception as e:
        logger.error(f'Erro ao protocolar pedidos: {e}')


def protocol_non_existent_order(order: Order) -> None:
    try:
        protocoled_order = api_okvendas.put_protocol_orders(order.order_id)
        if protocoled_order:
            logger.info(current_job + f' | Pedido {order.order_id} protocolado via api OkVendas')
        else:
            logger.warning(current_job + f' | Falha ao protocolar pedido {order.order_id} via api OkVendas')
    except Exception as ex:
        logger.error(f'Erro ao protocolar pedido {order.order_id}: {str(ex)}')


def insert_temp_order(order: Order, db_config: DatabaseConfig) -> bool:
    step = ''
    db = database.Connection(db_config)
    conn = db.get_conect()
    try:
        step = 'conexao'
        cursor = conn.cursor()

        if order.user.erp_code is not None and order.user.erp_code != '':
            cursor.execute(queries.get_query_client_erp(db_config.db_type), queries.get_command_parameter(db_config.db_type, [order.user.erp_code]))
            existent_client = cursor.fetchone()
            if existent_client is None:
                # insere cliente
                step = 'insere cliente'
                logger.info(f'\tPedido {order.order_id}: Inserindo cliente')
                cursor.execute(queries.get_insert_client_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [
                    order.user.name,
                    order.user.company_name or order.user.name,
                    order.user.cpf,
                    order.user.cnpj,
                    order.user.email,
                    order.user.residential_phone,
                    order.user.mobile_phone,
                    order.user.address.zipcode,
                    order.user.address.address_type,
                    order.user.address.address_line,
                    order.user.address.number,
                    order.user.address.complement,
                    order.user.address.neighbourhood,
                    order.user.address.city,
                    order.user.address.state,
                    order.user.address.reference]))
            else:
                # update no cliente existente
                step = 'update cliente'
                logger.info(f'\tPedido {order.order_id}: Atualizando cliente existente')
                cursor.execute(queries.get_update_client_sql(db_config.db_type), queries.get_command_parameter(db_config.db_type, [
                    order.user.name,
                    order.user.company_name or order.user.name,
                    order.user.cpf,
                    order.user.cnpj,
                    order.user.email,
                    order.user.residential_phone,
                    order.user.mobile_phone,
                    order.user.address.zipcode,
                    order.user.address.address_type,
                    order.user.address.address_line,
                    order.user.address.number,
                    order.user.address.complement,
                    order.user.address.neighbourhood,
                    order.user.address.city,
                    order.user.address.state,
                    order.user.address.reference,
                    order.user.erp_code]))

            if cursor.rowcount > 0:
                logger.info(current_job + f' | Cliente inserido/atualizado para o pedido {order.order_id}')
                cursor.execute(queries.get_query_client(db_config.db_type), queries.get_command_parameter(db_config.db_type, [order.user.email]))
                client_id = cursor.fetchone()
                if client_id is None:
                    cursor.close()
                    raise Exception('Nao foi possivel obter o cliente inserido do banco de dados')
            else:
                cursor.close()
                raise Exception('O cliente nao foi inserido')

        # insere pedido
        step = 'insere pedido'
        logger.info(f'\tPedido {order.order_id}: Inserindo cabecalho pedido')
        cursor.execute(queries.get_insert_order_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [
            order.order_id,
            order.order_code,
            str(datetime.strptime(order.date.replace('T', ' '), '%Y-%m-%d %H:%M:%S')),
            order.status,
            client_id[0],
            order.total_amount,
            order.total_discount,
            order.freight_amount,
            order.additional_payment_amount,
            str(datetime.strptime(order.paid_date.replace('T', ' '), '%Y-%m-%d %H:%M:%S')),
            order.payment_type,
            order.flag,
            order.parcels,
            order.erp_payment_condition,
            order.tracking_code,
            str(datetime.strptime(order.delivery_forecast.replace('T', ' '), '%Y-%m-%d %H:%M:%S')),
            order.carrier,
            order.shipping_mode]))

        if cursor.rowcount > 0:
            logger.info(current_job + f' | Pedido {order.order_id} inserido')
            cursor.execute(queries.get_query_order(db_config.db_type), queries.get_command_parameter(db_config.db_type, [order.order_id]))
            order_id = cursor.fetchone()
            if order_id is None:
                cursor.close()
                raise Exception('Nao foi possivel obter o pedido inserido no banco de dados')
        else:
            cursor.close()
            raise Exception('O cliente nao foi inserido')

        # insere itens
        step = 'insere itens'
        logger.info(f'\tPedido {order.order_id}: Inserindo itens do pedido')
        for item in order.items:
            cursor.execute(queries.get_insert_order_items_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [
                order.order_id,
                item.sku,
                item.erp_code,
                item.quantity,
                item.ean,
                item.value,
                item.discount,
                item.freight_value]))

        cursor.close()
        conn.commit()
        conn.close()
        return True
    except Exception as e:
        logger.error(current_job + f' | Passo {step} - Erro durante a inserção dos dados do pedido {order.order_id}: {str(e)}')
        conn.rollback()
        conn.close()
        return False


def call_order_procedures(db_config: DatabaseConfig, order_id: int) -> (bool, str, str):
    client_erp_id = ''
    order_erp_id = ''
    success = True
    db = database.Connection(db_config)
    conn = db.get_conect()
    try:
        cursor = conn.cursor()

        client_out_value = cursor.var(str)
        cursor.callproc('OPENK_SEMAFORO.SP_PROCESSA_CLIENTE', [order_id, client_out_value])
        client_erp_id = client_out_value.getvalue()
        if client_erp_id is not None:
            logger.info(current_job + f' | Cliente ERP criado com sucesso {client_erp_id}')
            order_out_value = cursor.var(str)
            cursor.callproc('OPENK_SEMAFORO.SP_PROCESSA_PEDIDO', [order_id, order_out_value])
            order_erp_id = order_out_value.getvalue()
            if order_erp_id is not None:
                logger.info(current_job + f' | Pedido ERP criado com sucesso {order_erp_id}')
            else:
                success = False
                logger.warning(current_job + f' | Nao foi possivel obter o id do pedido do ERP (retorno da procedure)')
        else:
            success = False
            logger.warning(current_job + f' | Nao foi possivel obter o id do cliente do ERP (retorno da procedure)')

        cursor.close()
        if success:
            conn.commit()
        else:
            conn.rollback()
        conn.close()
        return success, client_erp_id, order_erp_id
    except Exception as e:
        logger.error(current_job + f' | Erro: {e}')
        conn.rollback()
        conn.close()


def call_update_order_procedure(db_config: DatabaseConfig, order: Order) -> bool:
    success = True
    db = database.Connection(db_config)
    conn = db.get_conect()
    try:
        cursor = conn.cursor()
        order_updated = cursor.var(int)
        cursor.callproc('OPENK_SEMAFORO.SP_ATUALIZA_PEDIDO', [order.order_id, order.status, order_updated])
        updated = order_updated.getvalue()
        if updated is None or updated <= 0:
            success = False
            logger.warning(current_job + f' | Nao foi possivel atualizar o pedido informado')

        cursor.close()
        if success:
            conn.commit()
        else:
            conn.rollback()
        conn.close()
        return success
    except Exception as e:
        logger.error(current_job + f' | Erro: {e}')
        conn.rollback()
        conn.close()


def check_order_existence(db_config: DatabaseConfig, order_id: int) -> bool:
    db = database.Connection(db_config)
    conn = db.get_conect()
    try:
        cursor = conn.cursor()
        cursor.execute(queries.get_query_order(db_config.db_type), queries.get_command_parameter(db_config.db_type, [order_id]))
        existent_order = cursor.fetchone()[0]
        cursor.close()
        conn.close()
        return existent_order is not None and existent_order > 0
    except Exception as e:
        conn.close()
        return False


def query_invoices(db_config: DatabaseConfig) -> list[Invoice]:
    """
    Consulta as notas fiscais a serem enviadas na api do okvendas
    Args:
        db_config: Configuracao do banco de dados

    Returns:
        Lista de notas fiscais para enviar
    """
    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    invoices = list[Invoice]
    try:
        cursor.execute(db_config.sql.replace(';', ''))
        rows = cursor.fetchall()
        columns = [col[0].lower() for col in cursor.description]
        results = [dict(zip(columns, row)) for row in rows]
        cursor.close()
        conn.close()
        if len(results) > 0:
            invoices = [Invoice(**p) for p in results]

    except Exception as ex:
        logger.error(f' Erro ao consultar notas fiscais no banco:  {ex}')

    return invoices


def query_trackings(db_config: DatabaseConfig) -> list[Tracking]:
    """
    Consulta os rastreios a serem enviados na api do okvendas
    Args:
        db_config: Configuracao do banco de dados

    Returns:
        Lista de notas fiscais para enviar
    """
    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    trackings = list[Tracking]
    try:
        cursor.execute(db_config.sql.replace(';', ''))
        rows = cursor.fetchall()
        columns = [col[0].lower() for col in cursor.description]
        results = [dict(zip(columns, row)) for row in rows]
        cursor.close()
        conn.close()
        if len(results) > 0:
            trackings = [Tracking(**p) for p in results]

    except Exception as ex:
        logger.error(f' Erro ao consultar rastreios no banco: {ex}')

    return trackings




import mysql.connector
import cx_Oracle
import pyodbc

from src.database.utils import DatabaseConfig
import src


class Connection:
    def __init__(self, db_config: DatabaseConfig):
        self.db_type = db_config.db_type.lower()
        self.db_config = db_config
        if self.db_type == 'mysql':
            self.conn = get_mysql_connection(db_config.db_host, db_config.db_name, db_config.db_user, db_config.db_pwd)
        elif self.db_type == 'oracle':
            self.conn = get_oracle_client_connection(db_config.db_client, db_config.db_user, db_config.db_pwd, db_config.db_host)
        elif self.db_type == 'sqlserver':
            self.conn = get_sql_server_connection(db_config.db_client, db_config.db_host, db_config.db_name, db_config.db_user, db_config.db_pwd)

    def get_conect(self):
        if self.db_type == 'oracle':
            return get_oracle_connection(self.db_config.db_host, self.db_config.db_user, self.db_config.db_pwd)
        else:
            return self.conn


def get_mysql_connection(db_host: str, db_name: str, db_user: str, db_pwd):
    return mysql.connector.connect(
        host=db_host,
        database=db_name,
        user=db_user,
        password=db_pwd)


def get_oracle_connection(db_host: str, db_user: str, db_pwd):
    conn_str = db_user + '/' + db_pwd + '@' + db_host
    return cx_Oracle.connect(conn_str)


def get_oracle_client_connection(db_client: str, db_user: str, db_pwd: str, db_host: str):
    if src.is_connected_oracle_client is False:
        cx_Oracle.init_oracle_client(lib_dir=rf'{db_client}')
        src.is_connected_oracle_client = True

    return cx_Oracle.connect(user=db_user,
                             password=db_pwd,
                             dsn=db_host)


def get_sql_server_connection(db_driver: str, db_host: str, db_name: str, db_user:str, db_pwd: str):
    return pyodbc.connect(f'DRIVER={{{db_driver}}};SERVER={db_host};DATABASE={db_name};UID={db_user};PWD={db_pwd}')
